#include "glew.h"
#include "freeglut.h"
#include "glm.hpp"
#include "ext.hpp"
#include <iostream>
#include <cmath>

#include "Shader_Loader.h"
#include "Render_Utils.h"

#include "Box.cpp"

// ZADANIE: Now, move the camera back from the box. Use the code for drawing cubes from the previous exercises.
// 
// You have to create a correct camera matrix (in the function createCameraMatrix). Send the compound transformation matrix aggragating both camera and projection transformations to the shader.
// Do not translate the box, it has to stay at the origin of the coordinate system.

GLuint program;
Core::Shader_Loader shaderLoader;

glm::mat4 createCameraMatrix()
{
	// Fill in the code. Calaculate the vectors fwd, up and side, and setup correctly the matrices cameraRotation and cameraTranslation.

	// The time variable will be used to have the cmaera rotate around the box with a radius 2.
	float time = glutGet(GLUT_ELAPSED_TIME) / 1000.0f;
	const float radius = 2.0f;
	glm::vec3 camPos = glm::vec3(cosf(time), 0, sinf(time)) * radius;

	// Since we want to look at the box sitting at the origin, the look-at-vector "fwd" should be inverse to the position vector of the camera.
	// Vector "up" is oriented up. Vector "side" is the cross product of the previous 2 vectors. The function glm::cross(vec3 a, vec3 b) returns the cross product of two vectors a and b.
	// The function glm::normalize(vec3 v) returns the unit vector with same direction as v.
	glm::vec3 fwd;
	glm::vec3 up;
	glm::vec3 side;

	// Remember about the assumption of orienting perspective such as to look in the negative z-direction.
	glm::mat4 cameraRotation;

	glm::mat4 cameraTranslation;
	
	return cameraRotation * cameraTranslation;
}

glm::mat4 createPerspectiveMatrix()
{
	const float zNear = 0.1f, zFar = 10.0f;
	const float frustumScale = 1.0f;
	glm::mat4 perspective;
	perspective[0][0] = frustumScale;
	perspective[1][1] = frustumScale;
	perspective[2][2] = (zFar + zNear) / (zNear - zFar);
	perspective[3][2] = (2 * zFar * zNear) / (zNear - zFar);
	perspective[2][3] = -1;
	perspective[3][3] = 0;

	return perspective;
}

void renderScene()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glClearColor(0.0f, 0.3f, 0.3f, 1.0f);

	glUseProgram(program);

	glm::mat4 camera = createCameraMatrix();
	glm::mat4 perspective = createPerspectiveMatrix();

	// Calculate the correct transformation matrix of the rendering pipeline to send to the GPU.
	glm::mat4 transformation;

	glUniformMatrix4fv(glGetUniformLocation(program, "transformation"), 1, GL_FALSE, (float*)&transformation);

	// Draw a box at the origin

	glUseProgram(0);
	glutSwapBuffers();
}

void init()
{
	glEnable(GL_DEPTH_TEST);
	program = shaderLoader.CreateProgram("shaders/shader_2_1.vert", "shaders/shader_2_1.frag");
}

void shutdown()
{
	shaderLoader.DeleteProgram(program);
}

void idle()
{
	glutPostRedisplay();
}

int main(int argc, char ** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(200, 200);
	glutInitWindowSize(600, 600);
	glutCreateWindow("OpenGL Pierwszy Program");
	glewInit();

	init();
	glutDisplayFunc(renderScene);
	glutIdleFunc(idle);

	glutMainLoop();

	shutdown();

	return 0;
}
